using System;
using System.Collections.Generic;
using System.Text;

namespace ToggleNic
{
    class Program
    {
        /// <summary>
        /// Indicates whether to toggle, or specifically enable/disable a connection
        /// </summary>
        enum ToggleMode
        {
            /// <summary>
            /// If the connection is enabled, it will be disabled, and vice versa.
            /// </summary>
            Toggle,
            /// <summary>
            /// If the connection is disabled, it will be enabled; otherwise nothing is done.
            /// </summary>
            Enable,
            /// <summary>
            /// If the connection is enabled, it will be disabled; otherwise nothing is done.
            /// </summary>
            Disable
        }

        static void Main(string[] args)
        {
            if( args.Length < 1 )
            {
                PrintUsage();
                return;
            }

            ToggleMode mode;
            string connectionName = args[0];
            if( args.Length == 1 )
                mode = ToggleMode.Toggle;
            else if( args.Length == 2 )
            {
                if( args[1] == "enable" )
                    mode = ToggleMode.Enable;
                else if( args[1] == "disable" )
                    mode = ToggleMode.Disable;
                else
                {
                    PrintUsage();
                    return;
                }
            }
            else
            {
                PrintUsage();
                return;
            }

            Shell32.Folder networkConnectionsFolder = GetNetworkConnectionsFolder();
            if( networkConnectionsFolder == null )
            {
                Console.WriteLine("Network connections folder not found.");
                return;
            }

            Shell32.FolderItem2 networkConnection = GetNetworkConnection(networkConnectionsFolder, connectionName);
            if( networkConnection == null )
            {
                Console.WriteLine("Network connection not found.");
                return;
            }

            Shell32.FolderItemVerb verb;
            try
            {
                if( IsNetworkConnectionEnabled(networkConnection, out verb) )
                {
                    if( mode == ToggleMode.Enable )
                        Console.WriteLine("Connection is already enabled.");
                    else
                    {
                        Console.WriteLine("Disabling connection...");
                        verb.DoIt();
                        System.Threading.Thread.Sleep(2000);
                    }
                }
                else
                {
                    if( mode == ToggleMode.Disable )
                        Console.WriteLine("Connections is already disabled.");
                    else
                    {
                        Console.WriteLine("Enabling connection...");
                        verb.DoIt();
                        System.Threading.Thread.Sleep(2000);
                    }
                }
            }
            catch( ArgumentException ex )
            {
                Console.WriteLine(ex.Message);
            }
        }

        /// <summary>
        /// Gets the Network Connections folder in the control panel.
        /// </summary>
        /// <returns>The Folder for the Network Connections folder, or null if it was not found.</returns>
        static Shell32.Folder GetNetworkConnectionsFolder()
        {
            Shell32.Shell sh = new Shell32.Shell();
            Shell32.Folder controlPanel = sh.NameSpace(3); // Control panel

            Shell32.FolderItems items = controlPanel.Items();
            foreach( Shell32.FolderItem item in items )
            {
                if( item.Name == Properties.Settings.Default.NetworkConnectionsFolderName )
                    return (Shell32.Folder)item.GetFolder;
            }

            return null;
        }

        /// <summary>
        /// Gets the network connection with the specified name from the specified shell folder.
        /// </summary>
        /// <param name="networkConnectionsFolder">The Network Connections folder.</param>
        /// <param name="connectionName">The name of the network connection.</param>
        /// <returns>The FolderItem for the network connection, or null if it was not found.</returns>
        static Shell32.FolderItem2 GetNetworkConnection(Shell32.Folder networkConnectionsFolder, string connectionName)
        {
            Shell32.FolderItems items = networkConnectionsFolder.Items();
            foreach( Shell32.FolderItem2 item in items )
            {
                if( item.Name == connectionName )
                {
                    return item;
                }
            }

            return null;
        }

        /// <summary>
        /// Gets whether or not the network connection is enabled and the command to enable/disable it.
        /// </summary>
        /// <param name="networkConnection">The network connection to check.</param>
        /// <param name="enableDisableVerb">On return, receives the verb used to enable or disable the connection.</param>
        /// <returns>True if the connection is enabled, false if it is disabled.</returns>
        static bool IsNetworkConnectionEnabled(Shell32.FolderItem2 networkConnection, out Shell32.FolderItemVerb enableDisableVerb)
        {
            Shell32.FolderItemVerbs verbs = networkConnection.Verbs();
            foreach( Shell32.FolderItemVerb verb in verbs )
            {
                if( verb.Name == Properties.Settings.Default.EnableVerbName )
                {
                    enableDisableVerb = verb;
                    return false;
                }
                else if( verb.Name == Properties.Settings.Default.DisableVerbName )
                {
                    enableDisableVerb = verb;
                    return true;
                }
            }

            throw new ArgumentException("No enable or disable verb found.");
        }

        static void PrintUsage()
        {
            Console.WriteLine("Usage: ToggleNic connection_name [enable|disable]");
            Console.WriteLine();
            Console.WriteLine("Specify enable to enable the NIC if it is disabled, otherwise leave it");
            Console.WriteLine("enabled. Specify disable to disable the NIC if it is enabled, otherwise");
            Console.WriteLine("leave it disabled. Specify no arguments to toggle the NIC.");
            Console.WriteLine();
            Console.WriteLine("Examples:");
            Console.WriteLine("ToggleNic \"Local Area Connection\"");
            Console.WriteLine("ToggleNic \"Local Area Connection 2\" enable");
            Console.WriteLine("ToggleNic \"Wireless Connection\" disable");
        }
    }
}